#!/usr/bin/perl

#Murasaki - multiple genome global alignment program
#Copyright (C) 2006-2007 Keio University
#(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use Getopt::Long;
use Pod::Usage;
use File::Basename;
#use Data::Dump qw {dump};

use strict;

our $root=(fileparse($0))[1];
$root=$ENV{MURASAKI} if $ENV{MURASAKI};
require "$root/config.pl";
our $seqhome="$root/seq";

my $output=pop(@ARGV);
if(!$output or (-f $output and formatCheck($output))){
  push(@ARGV,$output) if $output;
  $output=">-";
}
pod2usage({-verbose=>1,-exitval=>2,-message=>'Need some input file...'}) if $#ARGV<0;

open(our $outf,">$output") or die "Couldn't open $output for writing.";

foreach my $file (@ARGV){
  open(my $inf,$file);
  my $state=0;
  my @specs=guessSpecies($file);
  warn "Couldn't identify a species for columns..." if(grep(/\?/,@specs));
  my $kogname;
  while(<$inf>){
    if($state==0){ #example line: >5
      ($kogname)=m/^>(.*)/ or die "Bad line: $_";
      print $outf "[?] $kogname SPEED-derived kog\n";
      $state++;
    }else{
      my @members=split(/\s+/);
      die "Invalid number of members! $_" unless $#members==$#specs;
      foreach my $i (0..$#members){
	next unless $members[$i]=~m/[^-]/;
	print $outf "  $specs[$i]:  $members[$i]\n";
      }
      print $outf "\n";
      $state=0;
    }
  }
}

sub formatCheck {
  my $file=pop;
  open(my $inf,$file) or return undef;
  my ($line1,$line2)=map {readline($inf)} (0,0);
  return 1 if($line1=~m/^>\w+/ and $line2=~m/(\w+\s+)+/);
  return undef;
}

sub guessSpecies {
  my $file=pop;
  my @bits=split(/\./,$file);
  my $clue=$#bits>0 ? $bits[$#bits-1]:$file;
  return qw{hsa ptr mac mmu rno cfa bov pos avi} if $clue eq 'hcmmrdcoc'; #hacchy's alias for: human chimp macaq mouse rat dog cow oposs chick (note: i made up mac cow pos and avi)
  my @tokens=split(//,$clue);
  my %conv=(h=>'hsa',m=>'mmu',r=>'rno');
  my $unknown=0;
  return map {exists($conv{$_}) ? $conv{$_}:"??".(++$unknown)} @tokens;
}

__END__

=head1 NAME

speed2kog.pl - converts a Hacchy-style ripped SPEED database to kog format

=head1 SYNOPSIS

speed2kog.pl [options] <input1> [input2 ... ] [output file]

=head1 OPTIONS

If "output file" exists, and is of the right format, it is considered an input file.

No other option exist yet.
