/*
Murasaki - multiple genome global alignment program
Copyright (C) 2006-2007 Keio University
(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

///////////////
//dinkymath.cc
//for all those math functions you want all over and hate rewriting
///////////

#include "dinkymath.h"
#include "murasaki.h"
#include <iostream>
#include <fstream>
#include <vector>
#include <assert.h>
#include <iterator>
#include <stdlib.h>

using namespace std;

string littleEndianStr(unsigned int n,int digits){
  char str[200];
  unsigned int mask=1;
  int last=0;
  for(int i=0;mask;i++,mask<<=1){
    str[i]=(n & mask) ? '1':'0';
    if(n & mask)
      last=i;
  }
  str[digits]=0;
  return string(str);
}

string littleEndianPat(unsigned int n,int digits){
  char str[200];
  unsigned int mask=1;
  int last=0;
  for(int i=0;mask;i++,mask<<=1){
    str[i]=(n & mask) ? '1':' ';
    if(n & mask)
      last=i;
  }
  str[digits]=0;
  return string(str);
}

string littleEndianStr(unsigned int n){
  return littleEndianStr(n,32);
}

template <class T>
int writeOutVector(string filename,const vector<T> &v,string delim,int from,int to,int skipZeros){
  ofstream os(filename.c_str());
  for(int i=from;i<to;i++)
   {
     if(v[i] || !skipZeros)
       os << i << delim << v[i] << endl;
   }
  return 0;
}

string dstring(int i){
  char buf[50];
  sprintf(buf,"%d",i);
  return string(buf);
}

string fstring(float f){
  char buf[50];
  sprintf(buf,"%.2f",f);
  return string(buf);
}

string humanMemory(unsigned int bytes){
  return humanMemory((unsigned long) bytes);
}
string humanMemory(int bytes){
  return humanMemory((unsigned long) bytes);
}

string humanMemory(unsigned long b){
  char buf[64];
  float f=(float)b;
  if(b==1)
    return string("1 byte");
  if(b<1024)
    sprintf(buf,"%lu bytes",b);
  else if(b<1024*1024)
    sprintf(buf,"%.2f kb",f/1024);
  else if(b<1024*1024*1024)
    sprintf(buf,"%.2f mb",f/1024/1024);
  else if(f<1024.*1024.*1024.*1024.)
    sprintf(buf,"%.2f gb",f/1024/1024/1024);
  else
    sprintf(buf,"%.3f tb",f/1024/1024/1024/1024);
  return string(buf);
}

string humanMemory(double f){
  char buf[64];
  if(f==1)
    return string("1 byte");
  if(f<1024)
    sprintf(buf,"%f bytes",f);
  else if(f<1024*1024)
    sprintf(buf,"%.1f kb",f/1024);
  else if(f<1024*1024*1024)
    sprintf(buf,"%.1f mb",f/1024/1024);
  else if(f<1024.*1024.*1024.*1024.)
    sprintf(buf,"%.1f gb",f/1024/1024/1024);
  else if(f<1024.*1024.*1024.*1024.*1024.)
    sprintf(buf,"%.3f tb",f/1024/1024/1024/1024);
  else
    sprintf(buf,"%.3e bytes",f/1024/1024/1024/1024);
  return string(buf);
}

ProgressTicker::ProgressTicker(int _max):
  max(_max),lineDelay(max/10),dotDelay(max/100),
  lineCount(0),dotCount(0)
{ }

void ProgressTicker::reset(int _max){
  max=_max;
  lineDelay=max/10;
  dotDelay=max/100;
  lineCount=0;
  dotCount=0;
}

int ProgressTicker::tick(int i){
  int percent=dotDelay==0 ? i*100/max:(i/dotDelay);
  if(!lineCount--){
    dotCount=dotDelay;
    lineCount=lineDelay;
    char buf[50];
    sprintf(buf,"%2d%%",percent);
    if(percent==0)
      cout << buf;
    else
      cout << endl << buf;
    cout.flush();
    return percent; 
  }
  if(!dotCount--){
    dotCount=dotDelay;
    cout << ".";
    cout.flush();
    return percent;
  }
  return -1; //no tick
}

int progressTick(int i,int max){
  static int lineDelay=max/10,dotDelay=max/100,
    lineCount=0,dotCount=0;
  int percent=dotDelay==0 ? i*100/max:(i/dotDelay);
  if(!lineCount--){
    dotCount=dotDelay;
    lineCount=lineDelay;
    char buf[50];
    sprintf(buf,"%2d%%",percent);
    cout << endl << buf;
    cout.flush();
    return percent; 
  }
  if(!dotCount--){
    dotCount=dotDelay;
    cout << ".";
    cout.flush();
    return percent;
  }
  return -1; //no tick
}

RandomTraverse::RandomTraverse(int _min,int _max) :
  min(_min),max(_max)
{
  srand(time(0));

  int mid=(min+max)/2;
  for(int i=0;i<mid;i++){
    hit.insert(mid+i);
    hit.insert(mid-i);
  }
  hit.insert(min);
  hit.insert(max);
}

int RandomTraverse::next(){
  if(hit.empty())return -1;

  int n=rand()%(*(--hit.end())-min)+min;
  if(!hit.count(n)){
    set<int>::iterator next=hit.upper_bound(n);
    assert(next!=hit.end());
    n=*next;
  }
  hit.erase(n);
  return n;
}

int RandomTraverse::remaining(){
  return hit.size();
}

void writeOut(string filename,string content){
  ofstream outf(filename.c_str());
  outf << content;
}
